@@ -3,7 +3,9 @@
 
 #include <QGraphicsItem>
 #include <QMap>
+#include <QTextLayout>
 
+class QTextLayout;
 class TMapRoute;
 class TMapView;
 
@@ -31,6 +33,7 @@ public:
     int Height;
     TMapRoute *Route;
     NodeType TypeNode;
+    QTextLayout *TextLayout;
 
     enum InfoItemType{itText,itImage};
     struct InfoItem
