@@ -35,6 +35,12 @@
 #include "maplayer.h"
 #include "mapelement.h"
 
+//#if defined(Q_WS_MAEMO_5)
+//#include <QtGui/QX11Info>
+//#include <X11/Xlib.h>
+//#include <X11/Xatom.h>
+//#endif
+
 QTime testTimer;
 
 qreal INFINIT=1000000000;
@@ -44,13 +50,14 @@ const QPainter::RenderHints NonAntialiasing=QPainter::NonCosmeticDefaultPen;
 QString ru="ru";
 
 TMapView::TMapView(QString AppPath, QString CachePath) :
-  nodeBegin(NULL),nodeEnd(NULL),mapWidth(0),mapHeight(0),CachedMap(NULL),
-  nodeClick(NULL),mapFade(NULL),nodeBeginID(-1),nodeEndID(-1)
+  CachedMap(NULL),nodeBeginID(-1),nodeEndID(-1),
+  nodeBegin(NULL),nodeEnd(NULL),nodeClick(NULL),
+  mapFade(NULL),mapWidth(0),mapHeight(0)
 {
  hide();
  setAttribute(Qt::WA_OpaquePaintEvent);
  setAttribute(Qt::WA_NoSystemBackground);
- setAttribute(Qt::WA_PaintOnScreen);
+ //setAttribute(Qt::WA_PaintOnScreen);
 
 #if OPENGL_ENABLE //defined(Q_WS_MAEMO_5) //defined(Q_OS_WIN32) //!(defined Q_OS_WINCE || defined Q_OS_SYMBIAN)
  if(QGLFormat::hasOpenGL())
@@ -67,6 +74,18 @@ TMapView::TMapView(QString AppPath, QString CachePath) :
  config.MobileVersion=0;
 #endif
 
+#ifdef Q_WS_MAEMO_5
+ setAttribute(Qt::WA_Maemo5NonComposited);
+// if(winId()) // grab zoom keys - dont work... dont know why :(
+// {
+//  unsigned long val = 1; //grab
+//  Atom atom = XInternAtom(QX11Info::display(), "_HILDON_ZOOM_KEY_ATOM", False);
+//  if(atom)
+//   XChangeProperty(QX11Info::display(),winId(),atom,XA_INTEGER,32,
+//                   PropModeReplace,reinterpret_cast<unsigned char *>(&val),1);
+// }
+#endif
+
  config.locale=QLocale::system().name().section("_",0,0);
  initTranslit();
 
@@ -89,6 +108,7 @@ TMapView::TMapView(QString AppPath, QString CachePath) :
  setResizeAnchor(AnchorViewCenter);
  
  state.mapCached=0;
+ state.mapDragging=0;
  params.delayIndex=params.delayMetro=0;
  params.appPath=AppPath;
  params.mapCachePath=CachePath;
@@ -98,24 +118,16 @@ TMapView::TMapView(QString AppPath, QString CachePath) :
  TimerPopup.setInterval(500);
  TimerPopup.setSingleShot(1);
  connect(&TimerPopup, SIGNAL(timeout()), this, SLOT(popupByTimer()));
- 
- nodeAInfo = new QAction(QIcon(":/img/dialog-information.png"),tr("Information"),this);
- nodePopup = new QMenu();
- nodePopup->addAction(nodeAInfo);
- nodePopup->addSeparator();
- nodePopup->addAction(tr("Start"));
- nodePopup->addAction(tr("Finish"));
- connect(nodePopup->actions().at(2),SIGNAL(triggered()),this,SLOT(setNodeBegin()));
- connect(nodePopup->actions().at(3),SIGNAL(triggered()),this,SLOT(setNodeEnd()));
+ TimerDragging.setInterval(800);
+ TimerDragging.setSingleShot(1);
+ connect(&TimerDragging, SIGNAL(timeout()), this, SLOT(mapStopDragging()));
  
  QPalette palette;
  palette.setColor(backgroundRole(), Qt::white);
  setPalette(palette);
 
- connect(horizontalScrollBar(),SIGNAL(sliderPressed()),this,SLOT(mapStartDragging()));
- connect(verticalScrollBar(),SIGNAL(sliderPressed()),this,SLOT(mapStartDragging()));
- connect(horizontalScrollBar(),SIGNAL(sliderReleased()),this,SLOT(mapStopDragging()));
- connect(verticalScrollBar(),SIGNAL(sliderReleased()),this,SLOT(mapStopDragging()));
+ connect(horizontalScrollBar(),SIGNAL(sliderMoved(int)),this,SLOT(mapStartDragging()));
+ connect(verticalScrollBar(),SIGNAL(sliderMoved(int)),this,SLOT(mapStartDragging()));
 }
 
 void TMapView::mapStartDragging()
@@ -124,10 +136,13 @@ void TMapView::mapStartDragging()
  {
   if(config.enableLowqDrag && config.enableAntialiasing)
   {
+   //params.nodeFont.setStyleStrategy(QFont::NoAntialias);
+   //params.nodeFont.setStyleStrategy(QFont::PreferDefault);
    setRenderHints(NonAntialiasing);
   }
   state.mapDragging=1;
  }
+ TimerDragging.start();
 }
 
 void TMapView::mapStopDragging()
@@ -136,7 +151,11 @@ void TMapView::mapStopDragging()
  {
   if(config.enableLowqDrag)
   {
-   if(config.enableAntialiasing) setRenderHints(Antialiasing);
+   if(config.enableAntialiasing)
+   {
+    setRenderHints(Antialiasing);
+    foreach(TMapNode *node,nodeByID) node->update(); // dont need in 4.7.2 ???
+   }
    //else
   }
   state.mapDragging=0;
@@ -154,9 +173,13 @@ void TMapView::keyPressEvent(QKeyEvent *event)
     {
     case Qt::Key_Equal:
     case Qt::Key_Plus:
+    case Qt::Key_ZoomIn:
+    case Qt::Key_F7:
         scaleMap(qreal(1.2));
         break;
     case Qt::Key_Minus:
+    case Qt::Key_ZoomOut:
+    case Qt::Key_F8:
         scaleMap(1/qreal(1.2));
         break;
     case '*':
@@ -169,7 +192,17 @@ void TMapView::keyPressEvent(QKeyEvent *event)
 
 void TMapView::wheelEvent(QWheelEvent *event)
 {
- scaleMap(pow(qreal(2), event->delta()/qreal(240)));
+ if(event->buttons()==Qt::MidButton)
+ {
+  int offset=event->delta();
+  if (event->orientation() == Qt::Horizontal)
+   horizontalScrollBar()->setValue(horizontalScrollBar()->value()-offset);
+  else
+   verticalScrollBar()->setValue(verticalScrollBar()->value()-offset);
+ }
+ else
+  scaleMap(pow(qreal(2), event->delta()/qreal(240)));
+
 }
 
 void TMapView::drawBackground(QPainter *painter, const QRectF &rect)
@@ -186,6 +219,11 @@ void TMapView::drawBackground(QPainter *painter, const QRectF &rect)
     else
     {
      if(!mapWidth || !mapHeight) drawPaintCmd(painter,paintBkg);
+     if(!mapWidth || !mapHeight)
+     {
+      mapWidth=sceneRect().width()-100;
+      mapHeight=sceneRect().height()-100;
+     }
      QRect rect(0,0,mapWidth,mapHeight);
      if(CachedMap) delete CachedMap;
      CachedMap = new QPixmap(mapWidth,mapHeight);
@@ -246,8 +284,22 @@ void TMapView::drawBackground(QPainter *painter, const QRectF &rect)
 //   }
 //  }
 
-
+  //state.mapLoaded=0;
  }
+// else
+// {
+//  if(layout()) layout().
+//  int size=64;
+//  QString text=" "+window()->windowTitle();
+//  QFont font("Tahoma");
+//  font.setPixelSize(size);
+//  QFontMetrics fm(font);
+//  int length=fm.width(text);
+//  QPointF point=rect.center()-QPoint(length/2-size,0);
+//  painter->setFont(font);
+//  painter->drawPixmap(point-QPoint(size,size),window()->windowIcon().pixmap(size,size));
+//  painter->drawText(point,text);
+// }
 }
 //void TMapView::drawForeground(QPainter *painter, const QRectF &rect)
 //{
@@ -256,6 +308,7 @@ void TMapView::drawBackground(QPainter *painter, const QRectF &rect)
 void TMapView::scaleMap(qreal scaleFactor)
 {
  if(!state.mapLoaded) return;
+ mapStartDragging();
  //if(config.enableAntialiasing) setRenderHints(NonAntialiasing);
  params.mapZoomFactor = matrix().scale(scaleFactor, scaleFactor).m11(); //mapRect(QRectF(0, 0, 1, 1)).width();
  if (params.mapZoomFactor < qreal(0.1) || params.mapZoomFactor > qreal(20)) return;
@@ -280,14 +333,15 @@ void TMapView::zoomOut()
 void TMapView::fitMap()
 {
  if(!state.mapLoaded) return;
-
+ QRectF RectView=QRectF(0,0,mapWidth,mapHeight); // scene->sceneRect() without frame
+ mapStartDragging();
  if(config.MobileVersion)
  {
-  fitInView(scene->sceneRect(),Qt::KeepAspectRatioByExpanding);
+  fitInView(RectView,Qt::KeepAspectRatioByExpanding);
   verticalScrollBar()->setValue(0);
  }
  else
-  fitInView(scene->sceneRect(),Qt::KeepAspectRatio);
+  fitInView(RectView,Qt::KeepAspectRatio);
 
  params.mapZoomFactor=matrix().m11();
 }
@@ -421,9 +475,14 @@ void TMapView::loadMap(QString fileName)
 
 void TMapView::exportMap(QString fileName,QString ext)
 {
+ if(!mapWidth || !mapHeight)
+ {
+  mapWidth=sceneRect().width()-100;
+  mapHeight=sceneRect().height()-100;
+ }
  QRectF rect(0,0,mapWidth, mapHeight);
  QImage image(mapWidth, mapHeight, QImage::Format_ARGB32_Premultiplied);
- image.fill(NULL);
+ image.fill(0);
  QPainter painter(&image);
  painter.setRenderHints(Antialiasing);
  drawBackground(&painter,scene->sceneRect());
@@ -525,7 +584,10 @@ void TMapView::loadMapImageToPaintCmd(QString imageFile, QList<TPaintCmd> &paint
             if(type=="ARROW")       eType=dtArrow; else
              if(type=="STAIRS")      eType=dtStairs; else
               if(type=="RAILWAY")     eType=dtRailway; else
-               if(type=="SPOTCIRCLE")  eType=dtSpotCircle;
+               if(type=="SPOTCIRCLE")  eType=dtSpotCircle; else
+                if(type=="SPOTRECT")  eType=dtSpotRect; else
+                 if(type=="DASHED")  eType=dtDashed; else
+                  if(type=="OPAQUE")eType=dtOpaque;
   else continue;
 
   addPaintCmd(eParam,eType,paintCmd);
@@ -592,22 +654,65 @@ void TMapView::addPaintCmd(const QString &eParam,DrawType eType,QList<TPaintCmd>
    paintCmd << cmd;
   }break;
 
+ case dtOpaque:{cmd.nums << eParam.toInt()/100.0;  paintCmd << cmd;} break;
+
+ case dtDashed:
  case dtEllipse:
  case dtStairs:
  case dtArrow:
  case dtSpline:
  case dtLine:
+ case dtRailway:
  case dtPolygon:{
    QList<qreal> nums;
    nums.clear();
    foreach (QString str, eParam.split(",")) nums << str.toFloat();
+   if(eType==dtRailway)
+   {
+    cmd.nums << nums.at(0) << nums.at(1) << nums.at(2);;
+    nums.removeFirst();
+    nums.removeFirst();
+    nums.removeFirst();
+   }
    for(int i=0;i<nums.count();i=i+2)
    {
     if(i+1==nums.count()) break;
     cmd.polygon << QPointF(nums.at(i),nums.at(i+1));
    }
-   if(eType==dtSpline)
+
+   // PEN
+   cmd.pen.setWidth(1);
+   cmd.pen.setStyle(Qt::SolidLine);
+   if(nums.count()%2==1)
+    cmd.pen.setWidthF(nums.at(nums.count()-1));
+   if(eType==dtDashed) cmd.pen.setStyle(Qt::DashLine);
+
+   // TYPE
+   if(eType==dtSpline || eType==dtRailway)
    {
+    if(eType==dtRailway)
+    {
+     QPointF p1=cmd.polygon.at(0),p2=cmd.polygon.at(1);
+     int sign,Offset=cmd.nums.at(0)/2;
+     if(p1.y()>p2.y()) // WARNING: ANYKEY-METHOD! ;)
+     {
+      if(p1.x()>p2.x()) cmd.polygon.translate(-Offset,Offset);
+      else cmd.polygon.translate(Offset,Offset); // 3
+     }
+     else
+     {
+      if(p1.x()>p2.x()) cmd.polygon.translate(Offset,-Offset);//2
+      else cmd.polygon.translate(-Offset,-Offset); //1
+     }
+     qreal PenWidth=cmd.nums.at(1);
+     QVector<qreal> dashPattern;
+     cmd.nums.replace(0,cmd.nums.at(0)/(cmd.nums.at(0)+cmd.nums.at(1)+cmd.nums.at(2))*PenWidth*2);
+     dashPattern << 1/cmd.nums.at(1) << 1/cmd.nums.at(2);
+     cmd.pen.setWidthF(PenWidth);
+     cmd.pen.setCapStyle(Qt::FlatCap);
+     cmd.pen.setStyle(Qt::CustomDashLine);
+     cmd.pen.setDashPattern(dashPattern);
+    }
     cmd.path=splinePath(cmd.polygon);
     cmd.polygon.clear();
    }
@@ -628,11 +733,6 @@ void TMapView::addPaintCmd(const QString &eParam,DrawType eType,QList<TPaintCmd>
     cmd.polygon.clear();
    }
 
-
-   cmd.pen.setWidthF(1);
-   if(nums.count()%2==1)
-    cmd.pen.setWidthF(nums.at(nums.count()-1));
-
    nums.clear();
    paintCmd << cmd;
   }break;
@@ -685,7 +785,7 @@ void TMapView::addPaintCmd(const QString &eParam,DrawType eType,QList<TPaintCmd>
 
    paintCmd << cmd;
   }break;
- case dtSpotCircle:{
+ case dtSpotCircle:{ // need transfer to element
    QStringList param = eParam.trimmed().split(",");
    qreal radius=((QString)param.at(0)).toFloat();
    QPointF t,b,point=QPointF(((QString)param.at(1)).toFloat(),((QString)param.at(2)).toFloat());
@@ -695,10 +795,17 @@ void TMapView::addPaintCmd(const QString &eParam,DrawType eType,QList<TPaintCmd>
    cmd.polygon << t << b;
    paintCmd << cmd;
   }break;
+ case dtSpotRect:{ // need transfer to element
+  QStringList param = eParam.trimmed().split(",");
+  cmd.polygon << QPointF(((QString)param.at(0)).toFloat(),((QString)param.at(1)).toFloat())
+              << QPointF(((QString)param.at(2)).toFloat(),((QString)param.at(3)).toFloat());
+  paintCmd << cmd;
+ }break;
  default: return; break;
  }
 }
 
+// DRAW
 void TMapView::drawPaintCmd(QPainter *p,QList<TPaintCmd> &paintCmd)
 {
  p->setPen(Qt::NoPen);
@@ -709,37 +816,62 @@ void TMapView::drawPaintCmd(QPainter *p,QList<TPaintCmd> &paintCmd)
   {
   case dtPenColor: p->setPen(cmd.pen); break;
   case dtBrushColor: p->setBrush(cmd.brush); break;
+  case dtOpaque: p->setOpacity(cmd.nums.at(0)); break;
 
-  case dtEllipse:
-  case dtStairs:
-  case dtArrow:
+  case dtPolygon:
   case dtSpline:
   case dtLine:
-  case dtPolygon:{
-    QPen pen = p->pen();
+  case dtStairs:
+  case dtArrow:
+  case dtEllipse:
+  case dtDashed:
+  case dtRailway:
+  {
+    QPen tmp,pen;
+    tmp=pen=p->pen();
     if(cmd.pen.widthF()>0)
     {
+     pen.setStyle(cmd.pen.style());
      pen.setWidthF(cmd.pen.widthF());
      p->setPen(pen);
     }else p->setPen(QPen(Qt::NoPen));
     if(cmd.type==dtPolygon)
       p->drawPolygon(cmd.polygon);
     else
-    if(cmd.type==dtLine)
+    if(cmd.type==dtLine || cmd.type==dtDashed)
       p->drawPolyline(cmd.polygon);
     else
-    if(cmd.type==dtSpline || cmd.type==dtArrow || cmd.type==dtStairs)
+    if(cmd.type==dtSpline || cmd.type==dtArrow || cmd.type==dtStairs || cmd.type==dtRailway)
     {
      QBrush brush = p->brush();
      p->setBrush(QBrush(Qt::NoBrush));
-     p->drawPath(cmd.path);
+     if(cmd.type==dtRailway)
+     {
+      p->setPen(QPen(QBrush(tmp.color()),cmd.nums.at(0)*1.3,Qt::SolidLine,Qt::FlatCap));
+      //p->drawPoint(cmd.polygon.at(0)+QPoint(2,2));
+      //p->drawRect(QRect(QPointF(cmd.polygon.at(0)).toPoint(),QSize(5,5)));
+      //qDebug() << cmd.nums.at(0);
+      p->drawPath(cmd.path);
+      p->setPen(QPen(QBrush(params.mapColorBack),cmd.nums.at(0)));
+      p->drawPath(cmd.path);
+
+      pen.setWidthF(cmd.pen.widthF());
+      pen.setCapStyle(cmd.pen.capStyle());
+      pen.setStyle(cmd.pen.style());
+      pen.setDashPattern(cmd.pen.dashPattern());
+      p->setPen(pen);
+      p->drawPath(cmd.path);
+      //tmp.setWidth(1);
+     }else
+      p->drawPath(cmd.path);
      p->setBrush(brush);
     }
     else
     if(cmd.type==dtEllipse)
       p->drawEllipse(QRectF(cmd.polygon.at(0),cmd.polygon.at(1)));
 
-    p->setPen(pen);
+    //qDebug() << tmp.widthF() << pen.widthF();
+    p->setPen(tmp);
    } break;
 
   case dtPath: p->drawPath(cmd.path); break;
@@ -765,13 +897,23 @@ void TMapView::drawPaintCmd(QPainter *p,QList<TPaintCmd> &paintCmd)
     }
     p->drawPixmap(0,0,*cmd.image);
    } break;
-   case dtSpotCircle:{
+   case dtSpotCircle:{ //## need transfer to element
     QPen pen=p->pen();
     p->setPen(Qt::NoPen);
     p->drawEllipse(QRectF(cmd.polygon.at(0),cmd.polygon.at(1)));
     p->setPen(pen);
    } break;
-   default: return; break;
+   case dtSpotRect:{ //## need transfer to element
+    QBrush brush=p->brush();
+    QPen pen=p->pen();
+    p->setBrush(QBrush(params.mapColorBack));
+    p->setPen(Qt::NoPen);
+    QSizeF size( ((QPointF)cmd.polygon.at(1)).x()+5,((QPointF)cmd.polygon.at(1)).y()+2);
+    p->drawRect(QRectF(cmd.polygon.at(0),size));
+    p->setBrush(brush);
+    p->setPen(pen);
+   }break;
+   default: break;
   }
  }
 }
@@ -1155,8 +1297,8 @@ void TMapView::addLayerToPaintCmd(TMapLayer *mapLayer,QList<TPaintCmd> &paintCmd
 // tCmd.brush=QBrush(Qt::SolidPattern);
 // tCmd.type=dtBrushColor;
 // paintCmd << tCmd;
- QVector<qreal> dashStyle;
- dashStyle << qreal(1.5) << qreal(0.5);
+ QVector<qreal> dashPattern;
+ dashPattern << qreal(1.5) << qreal(0.5);
 
  foreach(TMapRoute *route,mapLayer->route)
  {
@@ -1166,7 +1308,7 @@ void TMapView::addLayerToPaintCmd(TMapLayer *mapLayer,QList<TPaintCmd> &paintCmd
   QPen solid=QPen(route->color,route->width,Qt::SolidLine,Qt::SquareCap,Qt::RoundJoin);
   QPen dash=QPen(route->color,route->width*qreal(0.8),Qt::CustomDashLine,Qt::FlatCap,Qt::RoundJoin);
 
-  dash.setDashPattern(dashStyle);
+  dash.setDashPattern(dashPattern);
   cmd.pen=solid;
   cmd.type=dtPenColor;
   paintCmd << cmd;
@@ -1289,15 +1431,16 @@ void TMapView::addLayerToPaintCmd(TMapLayer *mapLayer,QList<TPaintCmd> &paintCmd
 void TMapView::mouseMoveEvent(QMouseEvent *event)
 {
  QGraphicsView::mouseMoveEvent(event);
- if(event->buttons()==Qt::LeftButton && !state.mapDragging )
+ if(event->buttons()==Qt::LeftButton)
  {
-  if(config.enableLowqDrag && config.enableAntialiasing)
-  {
-   //params.nodeFont.setStyleStrategy(QFont::NoAntialias);
-   //params.nodeFont.setStyleStrategy(QFont::PreferDefault);
-   setRenderHints(NonAntialiasing);
-  }
-  state.mapDragging=1;
+  mapStartDragging();
+//  if(config.enableLowqDrag && config.enableAntialiasing)
+//  {
+//   //params.nodeFont.setStyleStrategy(QFont::NoAntialias);
+//   //params.nodeFont.setStyleStrategy(QFont::PreferDefault);
+//   setRenderHints(NonAntialiasing);
+//  }
+//  state.mapDragging=1;
  }
  if(TimerPopup.isActive() && QLineF(nodeClickPos,event->pos()).length()>qreal(8.0)) TimerPopup.stop();
 }
@@ -1318,7 +1461,11 @@ void TMapView::mousePressEvent (QMouseEvent *event)
  QGraphicsView::mousePressEvent(event);
 
  nodeClickPos=event->pos();
- if(event->button()==Qt::MidButton) fitMap();
+ if(event->button()==Qt::MidButton)
+ {
+  mapStartDragging();
+  fitInView(QRectF(0,0,mapWidth,mapHeight),Qt::KeepAspectRatioByExpanding); //fitMap();
+ }
  if(event->button()==Qt::LeftButton || event->button()==Qt::RightButton)
  {
   bool sv=0;
@@ -1336,7 +1483,7 @@ void TMapView::mousePressEvent (QMouseEvent *event)
 void TMapView::popupByTimer()
 {
  setDragMode(QGraphicsView::NoDrag);
- nodePopup->popup(mapToGlobal(nodeClickPos));
+ if(nodePopup) nodePopup->popup(mapToGlobal(nodeClickPos));
  if(config.enableLowqDrag && config.enableAntialiasing)
   setRenderHints(Antialiasing);
  setDragMode(QGraphicsView::ScrollHandDrag);
@@ -1351,20 +1498,20 @@ void TMapView::mouseReleaseEvent(QMouseEvent *event)
  if(event->button()==Qt::LeftButton)
  {
   TimerPopup.stop();
-  if(state.mapDragging)
-  {
-   if(config.enableLowqDrag)
-   {
-    if(config.enableAntialiasing)
-    {
-     setRenderHints(Antialiasing);
-     //params.nodeFont.setStyleStrategy(QFont::PreferAntialias);
-    }
-//    else
-//     params.nodeFont.setStyleStrategy(QFont::PreferDefault);
-   }
-   state.mapDragging=0;
-  }
+//  if(state.mapDragging)
+//  {
+//   if(config.enableLowqDrag)
+//   {
+//    if(config.enableAntialiasing)
+//    {
+//     setRenderHints(Antialiasing);
+//     //params.nodeFont.setStyleStrategy(QFont::PreferAntialias);
+//    }
+////    else
+////     params.nodeFont.setStyleStrategy(QFont::PreferDefault);
+//   }
+//   state.mapDragging=0;
+//  }
  }
 
  if(QLineF(nodeClickPos,event->pos()).length()>8) return;
@@ -1376,14 +1523,13 @@ void TMapView::mouseReleaseEvent(QMouseEvent *event)
   {
    clearTrack();
    if(nodeClick) nodeClick->update();
-   viewport()->update();
   }
   return;
  }
 
  if(event->button()==Qt::RightButton)
  {
-  nodePopup->popup(mapToGlobal(event->pos()));
+  if(nodePopup) nodePopup->popup(mapToGlobal(event->pos()));
  }
 
  if(event->button()==Qt::LeftButton)
@@ -1402,6 +1548,7 @@ void TMapView::selectedNodesToFront(bool enable)
  if(!selectedNode.count()) return;
 
  QRectF rect=selectedNode.at(0)->mapToScene(selectedNode.at(0)->boundingRect()).boundingRect();;
+ //qDebug() << rect;
  int z=(enable ? 2:-1),nPadding=params.nodeWidth*2;
  foreach(TMapNode *node, selectedNode)
  {
@@ -1423,6 +1570,7 @@ void TMapView::selectedNodesToFront(bool enable)
   int wh=qMin(rect.width(),rect.height());
   nPadding*=(wh/nPadding<qreal(4.0) ? 10:3);
   rect.adjust(-nPadding,-nPadding,nPadding,nPadding);
+  //qDebug() << rect;
   fitInView(rect,Qt::KeepAspectRatio);
  }
 }
@@ -1438,8 +1586,7 @@ void TMapView::findTrack(TMapNode *begin,TMapNode *end)
   return;
  }
  if(!begin || begin->TypeNode==TMapNode::ntConstruct ||
-    begin==end || (end && end->TypeNode==TMapNode::ntConstruct)) return;
- //if(end) if(end->TypeNode==TMapNode::ntConstruct) return;
+    begin==end /*|| (!end && begin==nodeBegin)*/ || (end && end->TypeNode==TMapNode::ntConstruct)) return;
 
  if(end==NULL) // click node: begin or end
  {
@@ -1459,7 +1606,7 @@ void TMapView::findTrack(TMapNode *begin,TMapNode *end)
  selectedNodesToFront(0);
 
  // calc distances
- if(nodeClick!=nodeEnd)
+ if(begin!=nodeEnd)
  {
   int size=mapGraph.count();
   workGraph.resize(size);
@@ -1487,10 +1634,11 @@ void TMapView::findTrack(TMapNode *begin,TMapNode *end)
   dijkstra(nodeBeginID);
   mapDistance[nodeBeginID]=params.delayMetro*60;
 
-  if(config.enableWriteTime) viewport()->update(); //foreach(TMapNode *node,nodeByID) node->update();
+  if(config.enableWriteTime) viewport()->repaint(); //foreach(TMapNode *node,nodeByID) node->update();
 
  }
  if(!nodeEnd) return;
+ state.mapLoaded=0;
  // find track
  QList<int> id;
  int endID=nodeEndID;
@@ -1502,7 +1650,7 @@ void TMapView::findTrack(TMapNode *begin,TMapNode *end)
  }
 
  //qDebug() << id;
-
+ mapStartDragging();
  selectedNode.clear();
  foreach(int i,id) selectedNode << nodeByID[i];
 
@@ -1567,6 +1715,7 @@ void TMapView::findTrack(TMapNode *begin,TMapNode *end)
 
  selectedNodesToFront(1);
  //qDebug() << id;
+ state.mapLoaded=1;
 }
 
 void TMapView::clearTrack()
@@ -1584,6 +1733,7 @@ void TMapView::clearTrack()
  //if(nodeByID.count()) foreach(TMapNode *node,nodeByID) node->update();
  emit trackText("");
  if(config.MobileVersion) emit trackFinded(0);
+ viewport()->repaint();
 }
 
 
@@ -1693,6 +1843,7 @@ void TMapView::applyConfig()
  config.mapLastFile="";
 
  //viewport()->update();
+ foreach(TMapNode *node,nodeByID) node->update();
 }
 
 void TMapView::setDelayMetro(int delay)
@@ -1725,7 +1876,7 @@ int TMapView::strTimeToSec(const QString &strTime)
 
 QString TMapView::convertToTextWithTags(QString text)
 {
- QRegExp url("((mailto:|(news|(ht|f)tp(s?))://)[^,<\\s]*)");
+ QRegExp url("((mailto:|(news|(ht|f)tp(s?))://)[^<\\s]*)");
  QString TagText,br=" <br>";
 
  text.replace("\\n",br);
@@ -1736,9 +1887,10 @@ QString TMapView::convertToTextWithTags(QString text)
  while(index != -1)
  {
   QString StrUrl=url.cap();
+  if(StrUrl.right(1)==",") StrUrl.chop(1);
   int UrlLength=StrUrl.length();
 
-  TagText.replace(index+offset,UrlLength,QString("<a href=\"%1\">%1</a> ").arg(StrUrl));
+  TagText.replace(index+offset,UrlLength,QString("<a href=\"%1\">%2</a> ").arg(StrUrl).arg(QUrl::fromEncoded(StrUrl.toAscii()).toString()));
   offset+=UrlLength+16;
   StrUrl.fill('x');
   text.replace(index,UrlLength,StrUrl);
@@ -1758,7 +1910,7 @@ void TMapView::loadNodeInfo(TMapNode *node)
   INISettings Info(params.mapPath+infoFile,codePage);
   if(!Info.value("Options/AddToInfo",0).toInt()) continue;
 
-  QString Type=Info.value("Type").toUpper();
+  QString Caption,Type=Info.value("Type").toUpper();
 
   TMapNode::InfoItem Item;
   if(Type=="IMAGE") Item.Type=TMapNode::itImage; else Item.Type=TMapNode::itText;
@@ -1766,22 +1918,26 @@ void TMapView::loadNodeInfo(TMapNode *node)
   if(Item.Value.length()<5) continue;
   if(Item.Type==TMapNode::itText) Item.Value=convertToTextWithTags(Item.Value);
   Item.Value=Info.value("Options/StringToAdd").remove('\'')+Item.Value;
-  node->Info.insert(Info.value("Options/Caption").remove('\''),Item);
+  //qDebug()<< Item.Value;
+  Caption=Info.value("Options/Caption").remove('\'');
+  if(node->Info.contains(Caption))
+   Item.Value=((TMapNode::InfoItem)node->Info.value(Caption)).Value+"<br>"+Item.Value;
+  node->Info.insert(Caption,Item);
  }
 }
 
-void TMapView::setNodeBegin()
+void TMapView::setNodeBegin(TMapNode *Node)
 {
- //nodeBegin=nodeClick;
- findTrack(nodeClick,nodeEnd);
- //nodeClick->update();
+ if(!Node) Node=nodeClick;
+ nodeBeginID=-1;
+ if(nodeBegin) nodeBegin->update();
+ nodeBegin=NULL;
+ findTrack(Node,nodeEnd);
 }
-void TMapView::setNodeEnd()
+void TMapView::setNodeEnd(TMapNode *Node)
 {
-  //nodeEnd=nodeClick;
- findTrack(nodeBegin,nodeClick);
-
- //nodeClick->update();
+ if(!Node) Node=nodeClick;
+ findTrack(nodeBegin,Node);
 }
 
 void TMapView::saveToFile()
