@@ -15,8 +15,8 @@ class TMapEdge;
 
 
 enum DrawType
-{dtPenColor,dtBrushColor,dtLine,dtPolygon,dtSpline,dtEllipse,dtTextOut,dtAngleTextOut,
- dtPeOpaqueAngleTextOut,dtImage,dtRailway,dtStairs,dtSpotRect,dtAngle,dtTranslate,dtPath,dtArrow,dtSpotCircle};
+{dtPenColor,dtBrushColor,dtLine,dtDashed,dtPolygon,dtSpline,dtEllipse,dtTextOut,dtAngleTextOut,dtOpaque,
+ dtPeOpaqueAngleTextOut,dtImage,dtRailway,dtStairs,dtAngle,dtTranslate,dtPath,dtArrow,dtSpotCircle,dtSpotRect};
 
 struct TPaintCmd
 {
@@ -61,7 +61,6 @@ private:
 
     QList<AddCacheEdge*> edgeCache; // cache for additional nodes & transfers
     QHash<QChar, QString> BGN;
-    QTextCodec *codePage;
     QString WindowTitle;
 
     //void drawMapLayer(QPainter *painter, TMapLayer *mapLayer);
@@ -97,7 +96,7 @@ public:
     void initTranslit();
     void convertToTranslit(QString &text);
 
-    QTimer TimerPopup;
+    QTimer TimerPopup, TimerDragging;
 
     QList<TMapLayer*> layer;
     QList<TPaintCmd> paintBkg,paintMap;
@@ -199,7 +198,7 @@ public:
     TMapNode *nodeBegin,*nodeEnd,*nodeClick;
     TMapElement *mapFade;
 
-    QAction *nodeAInfo;
+    QTextCodec *codePage;
     QMenu *nodePopup;
 
 public slots:
@@ -208,8 +207,8 @@ public slots:
  void zoomIn();
  void zoomOut();
  void fitMap();
- void setNodeBegin();
- void setNodeEnd();
+ void setNodeBegin(TMapNode *Node=NULL);
+ void setNodeEnd(TMapNode *Node=NULL);
  void popupByTimer();
  void saveToFile();
  void findTrackByID(int ID) { findTrack(nodeByID[ID]); }
